import 'package:fis_common/logger/logger.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/language/measure_language.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/view/measure/measure_view_controller.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_image.dart';

/// 测量项页面
class LeftSiderSelectMeasure extends FStatefulWidget {
  const LeftSiderSelectMeasure({Key? key}) : super(key: key);

  @override
  FState<LeftSiderSelectMeasure> createState() => LeftSiderSelectMeasureState();
}

class LeftSiderSelectMeasureState extends FState<LeftSiderSelectMeasure> {
  final ScrollController scrollController = ScrollController();

  late final application = Get.find<IApplication>();
  late final measureHandler = Get.find<MeasureHandler>();

  /// 数据
  late final measureData = Get.find<MeasureDataController>();
  final playerController = Get.find<IPlayerController>() as VidPlayerController;

  /// 测量项控制器
  final measureMetaController = Get.find<MeasureMetaController>();
  // final topMeasureItem = Get.put<ITopMeasureItem>();

  /// 当前选中的测量项目
  late String activeName = measureData.getItemMetaList[0].name;

  /// 当前组合测量项
  ItemMeta? comboItemMeta;

  int? comboItemMetaActivedIndex;

  /// 是否展开组合测量项
  bool isCombo = false;

  late String fontFamily;

  void workingChildChanged(sender, int e) {
    comboItemMetaActivedIndex = e;
    setState(() {});
  }

  /// 切换测量项
  void changeItem(ItemMeta itemMeta) {
    activeName = itemMeta.name;
    try {
      application.switchItem(itemMeta);
      final item = application.activeMeasureItem!;
      if (item is ITopMeasureItem) {
        item.switchChild(0);
        item.workingChildChanged.addListener(workingChildChanged);
      }
    } catch (e) {
      logger.e("changeItem failed", e);
    }
    if (mounted) {
      setState(() {});
    }
  }

  /// 当前测量数据源变更
  void itemMetaList(Object s, dynamic e) {
    try {
      if (e != null) {
        setState(() {});
      }
    } catch (e) {
      debugPrint("callmethod fail " + e.toString());
    }
  }

  /// 获取当前图像的测量项，从第一帧取
  void currentFrameHandler(Object sender, VidUsImage e) async {
    List<String> getModes = [];
    for (var element in e.visuals[0].modes) {
      getModes.add(element.type.toString().split('.')[1]);
    }
    measureData.applicationModes = e.visuals[0].modes;
    measureData.currentMode = e.visuals[0].modes.first.type.name;
    var measureModeSelection = MeasureModeSelection(
      application.applicationName,
      application.categoryName,
      application.isThirdPart ? ['TPPTissue'] : getModes,
    );
    measureHandler.measureModeChanged = measureModeSelection;
    var measureApplicationDTO =
        await measureData.getMeasureApplication.call(measureModeSelection);
    if (measureApplicationDTO != null) {
      /// 模式版本
      measureData.measureApplicationVersion =
          measureApplicationDTO.version ?? '';
      measureData.availableModes = measureApplicationDTO.availableModes ?? [];

      measureMetaController.setAvailableModes(measureData.currentMode);

      activeName = measureData.getItemMetaList[0].name;
      // measureMetaController.measureConfig();
      setState(() {});
    }
    getNoteCommentsList();
  }

  /// 注释获取
  void getNoteCommentsList() async {
    List<String> commentsList = [];
    var measureCommentItemResult =
        await measureData.getCommentsByApplicationAsync(
      application.applicationName,
      application.categoryName,
    );
    measureData.measureCommentItemResult =
        measureCommentItemResult?.commentItems ?? [];

    measureCommentItemResult?.commentItems?.forEach((element) {
      commentsList.add(element.text ?? '');
    });
    measureData.getCommentsList = commentsList;
  }

  void getItemMetaListChanged(sender, e) {
    print('getItemMetaList changed');
    isCombo = false;
    measureData.getTopItemMetaList = measureData.getItemMetaList;
    measureData.getBottomItemMetaList = [];
    if (measureData.getItemMetaList.isNotEmpty) {
      changeItem(measureData.getItemMetaList[0]);
    }
    setState(() {});
  }

  @override
  void initState() {
    playerController.currentFrameHandler.addListener(
      (currentFrameHandler),
    );
    measureData.getItemMetaListChanged.addListener(getItemMetaListChanged);
    measureData.itemMetaListChanged.addListener(itemMetaList);

    super.initState();
  }

  @override
  dispose() {
    super.dispose();
    playerController.currentFrameHandler.removeListener(
      (currentFrameHandler),
    );
    measureData.getItemMetaListChanged.removeListener(getItemMetaListChanged);
    measureData.itemMetaListChanged.removeListener(itemMetaList);

    final item = application.activeMeasureItem!;
    if (item is ITopMeasureItem) {
      item.workingChildChanged.removeListener(workingChildChanged);
    }
  }

  @override
  FWidget build(BuildContext context) {
    fontFamily = Theme.of(context).textTheme.button!.fontFamily!;
    return FSingleChildScrollView(
      controller: ScrollController(),
      child: FScrollbar(
        isAlwaysShown: true,
        child: FColumn(
          children: [
            _buildFGridView(measureData.getTopItemMetaList),
            if (isCombo) _buildCombo(),
            _buildFGridView(measureData.getBottomItemMetaList),
          ],
        ),
      ),
    );
  }

  /// 未选中的普通测量项 选中事件
  void chooseMeasure(ItemMeta itemMeta) {
    isCombo = false;
    changeItem(itemMeta);
    measureData.getTopItemMetaList = measureData.getItemMetaList;
    measureData.getBottomItemMetaList = [];
  }

  void chooseComboMeasure(ItemMeta itemMeta) {
    isCombo = true;
    changeItem(itemMeta);
    List<ItemMeta> top = [];
    List<ItemMeta> bottom = [];
    List<ItemMeta> getItemMetaList = measureData.getItemMetaList;
    for (int i = 0; i < getItemMetaList.length; i++) {
      int indexOfItemMeta = getItemMetaList.indexOf(itemMeta);
      if (i <= indexOfItemMeta && indexOfItemMeta % 2 != 0) {
        top.add(getItemMetaList[i]);
      } else if (i > indexOfItemMeta && indexOfItemMeta % 2 != 0) {
        bottom.add(getItemMetaList[i]);
      }
      if (indexOfItemMeta % 2 == 0 && i <= indexOfItemMeta + 1) {
        top.add(getItemMetaList[i]);
      }
      if (indexOfItemMeta % 2 == 0 && i > indexOfItemMeta + 1) {
        bottom.add(getItemMetaList[i]);
      }
      measureData.getTopItemMetaList = top.toSet().toList();
      measureData.getBottomItemMetaList = bottom.toSet().toList();
      if (i == getItemMetaList.indexOf(itemMeta)) {
        comboItemMeta = getItemMetaList[i];
        comboItemMetaActivedIndex = 0;
      }
    }
  }

  // String getValue(String code) {
  //   return MeasureLanguage().languageValue(code) ?? code;
  // }

  FWidget _buildComboItem(ItemMeta itemMeta, int index) {
    FWidget child;
    if (comboItemMetaActivedIndex == index) {
      child = FElevatedButton(
        onPressed: () {
          final item = application.activeMeasureItem!;
          if (item is ITopMeasureItem) {
            item.switchChild(index);
            comboItemMetaActivedIndex = index;
          }
          setState(() {});
        },
        child: FText(
          itemMeta.name,
          style: const TextStyle(
            color: Colors.white,
          ),
        ),
      );
    } else {
      child = FOutlinedButton(
        onPressed: () {
          final item = application.activeMeasureItem!;
          if (item is ITopMeasureItem) {
            item.switchChild(index);
            comboItemMetaActivedIndex = index;
          }
          setState(() {});
        },
        style: OutlinedButton.styleFrom(
          side: const BorderSide(
            color: Color.fromRGBO(124, 124, 124, 1),
          ),
        ),
        child: FText(
          itemMeta.name,
          style: const TextStyle(
            color: Colors.white,
          ),
        ),
      );
    }
    return child;
  }

  /// 组合测量项打开的ui
  FWidget _buildCombo() {
    return FColumn(
      mainAxisSize: MainAxisSize.max,
      children: [
        FContainer(
          padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 5),
          color: const Color.fromRGBO(70, 70, 70, 1),
          child: FColumn(
            children: (comboItemMeta?.childItems ?? [])
                .asMap()
                .entries
                .map(
                  (e) => FContainer(
                    width: double.infinity,
                    padding: const EdgeInsets.symmetric(
                      vertical: 2,
                    ),
                    child: _buildComboItem(
                      e.value,
                      e.key,
                    ),
                  ),
                )
                .toList(),
          ),
        ),
      ],
    );
  }

  FWidget _buildFGridView(List<ItemMeta> itemMeta) {
    return FGridView.count(
      shrinkWrap: true,
      crossAxisCount: 2,
      childAspectRatio: 2.5,
      controller: ScrollController(),
      children: itemMeta.map((e) {
        if (e.name == activeName) {
          // 选中的测量项
          return _buildItemMetaContainer(e, () {});
        } else {
          if (e.childItems.isEmpty) {
            return _buildItemMetaContainer(
              e,
              () => chooseMeasure(e),
            );
          } else {
            return _buildItemMetaContainer(
              e,
              () => chooseComboMeasure(e),
            );
          }
        }
      }).toList(),
    );
  }

  FWidget _buildItemMetaContainer(ItemMeta e, Function onTap) {
    FWidget measure;
    if (e.name == activeName) {
      measure = FElevatedButton(
        onPressed: () => onTap.call(),
        child: i18nBook.isCurrentChinese && MeasureLanguage.t(e.name) != e.name
            ? FColumn(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  FText(
                    MeasureLanguage.t(e.name),
                    maxLines: 1,
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      fontFamily: fontFamily,
                      fontSize: 12,
                    ),
                  ),
                  FText(
                    e.name,
                  ),
                ],
              )
            : FText(
                e.name,
              ),
        style: ElevatedButton.styleFrom(
          fixedSize: const Size.fromHeight(
            50,
          ),
        ),
      );
    } else {
      measure = FOutlinedButton(
        onPressed: () => onTap.call(),
        child: i18nBook.isCurrentChinese && MeasureLanguage.t(e.name) != e.name
            ? FColumn(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  FText(
                    MeasureLanguage.t(e.name),
                    maxLines: 1,
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      color: Colors.white,
                      fontFamily: fontFamily,
                      fontSize: 12,
                    ),
                  ),
                  FText(
                    e.name,
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ],
              )
            : FText(
                e.name,
                style: const TextStyle(
                  color: Colors.white,
                ),
              ),
        style: OutlinedButton.styleFrom(
          fixedSize: const Size.fromHeight(
            50,
          ),
          side: const BorderSide(
            color: Color.fromRGBO(124, 124, 124, 1),
          ),
          textStyle: const TextStyle(
            color: Colors.white,
          ),
        ),
      );
    }

    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: measure,
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }
}
